/***************************************************************************
 file:			game_defines.h
 description:	definitions of game and machine numbers
 copyright:		(C)1997-2002 Drunken Muppets (www.impactemu.com)
****************************************************************************/


#ifndef _GAME_DEFINES_H
#define _GAME_DEFINES_H



typedef enum 
{
	MACHINE_UNKNOWN = 0,
	MACHINE_CAPCOM_ZN1,
	MACHINE_CAPCOM_ZN2,
	MACHINE_NAMCO_S11,
	MACHINE_NAMCO_S12,
	MACHINE_BANPRESTO_KN1,
	MACHINE_RAIZING_ZN,
	MACHINE_TAITO_FX1A,
	MACHINE_TAITO_FX1A2,
	MACHINE_KONAMI_GV,
	MACHINE_KONAMI_GQ,
	MACHINE_KONAMI_DDR4,
	MACHINE_KONAMI_SYS573,
	MACHINE_NAMCO_S10,
	MACHINE_TECMO_TPS,
	MACHINE_ATARI_ZN,
	MACHINE_TAITO_FX1A3,
} vendor_id_t;

typedef enum 
{
	GAME_UNKNOWN = 0,
	GAME_DANCING_EYES,
	GAME_XEVIOUS_3DG,
	GAME_SOUL_EDGE_A,
	GAME_SOUL_EDGE_C,
	GAME_TEKKEN_1_A,
	GAME_TEKKEN_2_B,
	GAME_TEKKEN_TAG_B,
	GAME_STAR_SWEEP_A,
	GAME_QUIZ_MY_ANGEL_3_A,
	GAME_TOSHINDEN_2,
	GAME_TETRIS_THE_GRAND_MASTER,
	GAME_STAR_GLADIATOR_2,
	GAME_STREET_FIGHTER_EX,
	GAME_STREET_FIGHTER_EX_PLUS,
	GAME_STREET_FIGHTER_EX_2,
	GAME_STREET_FIGHTER_EX_2_PLUS,
	GAME_RIVAL_SCHOOLS,
	GAME_STRIDER_HIRYU_2,
	GAME_KIKAIOH,
	GAME_SONIC_WINGS_LIMITED,
	GAME_PSYCHIC_FORCE,
	GAME_MAGICAL_DATE,
	GAME_SUPER_FOOTBALL_CHAMP,
	GAME_FIGHTERS_IMPACT,
	GAME_RAYSTORM,
	GAME_BEASTORIZER,
	GAME_BEASTORIZER_ORG,
	GAME_DUNK_MANIA,
	GAME_GOLGO_13,
	GAME_QUIZ_MY_DREAM_HORSE,
	GAME_TEKKEN_3,
	GAME_FIGHTING_LAYER,
	GAME_SOUL_CALIBUR,
	GAME_PRIME_GOAL_EX,
	GAME_EHRGEIZ,
	GAME_MRDRILLER,
	GAME_PACA_PACA_PASSION_SPECIAL,
	GAME_AQUA_RUSH,
	GAME_SUPER_WORLD_STADIUM_99,
	GAME_MR_DRILLER_2,
	GAME_WEDDING_RHAPSODY,
	GAME_HYPER_ATHLETE,
	GAME_POWERFULL_BASEBALL_96,
	GAME_SUSUME_TAISEN_PUZZLE_DAMA,
	GAME_BEAT_THE_CHAMP,
	GAME_GALLOP_RACER,
	GAME_GALLOP_RACER2,
	GAME_SHANGHAI,
	GAME_COOL_BOARDERS,
	GAME_DOAPP,
	GAME_TONDEMO,
	GAME_BRAVE_BLADE,
	GAME_PRIMALRAGE2,
	GAME_STAR_GLADIATOR,
	GAME_BEASTORIZER2,
	GAME_PLASMA_SWORD,
	GAME_HEAVENS_GATE,
	GAME_G_DARIUS,
	GAME_G_DARIUS2,
	GAME_MONSTERFARMJUMP,
	GAME_GALLOP_RACER3,
	GAME_POINTBLANK2,
} game_id_t;


#endif // _GAME_DEFINES_H
