/***************************************************************************
 file:			controller_defines.h
 description:	definitions of jamma controller specific stuff
 copyright:		(C)1997-2002 Drunken Muppets (www.impactemu.com)
****************************************************************************/

#ifndef _CONTROLLER_DEFINES_H
#define _CONTROLLER_DEFINES_H

typedef struct
{
	unsigned long	controls;
	unsigned long	players[4];
	unsigned long	reserved[10];
} jamma_t;


#endif // _CONTROLLER_DEFINES_H